(* $Novell: addentry.c,v 1.11 2003/05/12 13:26:24 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    addentry.c
***************************************************************************
   Description: The addentry.c sample adds an entry ( cn=james ) with
                its mandatory attributes to a specified container.
***************************************************************************)
program addentry;

uses sysutils, ldap;

//static char usage[] =
//"\n Usage:   addentry <host name> <port number> <login dn> <password>"
//"\n\t  <continer name> \n"
//"\n Example: addentry Acme.com 389 cn=admin,o=Acme secret"
//" ou=Sales,o=Acme\n";

const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */

var
    version, ldapport, rc: int32;
    ldapHost  : AnsiString;
    loginDN   : AnsiString;
    password  : AnsiString;

    containerName: AnsiString;
    dn           : AnisString;

    vclass : array[0..4] of pchar8;
    vcn    : array[0..1] of pchar8;
    vgn    : array[0..1] of pchar8;
    vsn    : array[0..1] of pchar8;

    vphone : array[0..1] of pchar8;
    vmail  : array[0..1] of pchar8;
    vupname: array[0..1] of pchar8;
    vpass  : array[0..1] of pchar8;
    vsam   : array[0..1] of pchar8;

    ld     : hLDAP;

    modClass, modCN, modGN, modSN, modPhone, modMail, modupName, modPass, modSAM: LDAPMod;

    mods   : array[0..9] of pLDAPMod;


begin
    if (paramcount<>5) then begin
      writeln('Usage:   addentry <host name> <port number> <login dn> <password> <container name>');
      writeln('    Example: addentry Acme.com 389 cn=admin,o=Acme secret ou=Sales,o=Acme');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);
    containerName:= paramstr(5);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    ///*
    // * Initialize the LDAP session
    // */
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);

    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('LDAP session initialized');


    //* Bind to the server */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind successful');

    ///*
    // * To add an entry to the directory:
    // *   1. Create the array of LDAPMod structures representing the
    // *      attributes of the new entry
    // *   2. Specify the DN of the entry to create
    // *   3. Call ldap_add_ext_s to add the entry to the directory
    // */
    modClass.mod_op       :=   LDAP_MOD_ADD;
    modClass.mod_type     :=   'objectClass';
    modClass.mod_values   :=   ADDR(vclass);
    vclass[0]             :=   'top';
    vclass[1]             :=   'person';
    vclass[2]             :=   'organizationalPerson';
    vclass[3]             :=   'user';         //'inetOrgPerson'
    vclass[4]             :=   nil;

    modCN.mod_op          :=   LDAP_MOD_ADD;
    modCN.mod_type        :=   'cn';
    modCN.mod_values      :=   ADDR(vcn);
    vcn[0]                :=   'james';
    vcn[1]                :=   nil;

    modGN.mod_op          :=   LDAP_MOD_ADD;
    modGN.mod_type        :=   'givenName';
    modGN.mod_values      :=   ADDR(vgn);
    vgn[0]                :=   'James';
    vgn[1]                :=   nil;

    modSN.mod_op          :=   LDAP_MOD_ADD;
    modSN.mod_type        :=   'sn';
    vsn[0]                :=   'Smith';
    vsn[1]                :=   nil;
    modSN.mod_values      :=   ADDR(vsn);

    modPhone.mod_op       :=   LDAP_MOD_ADD;
    modPhone.mod_type     :=   'telephoneNumber';
    modPhone.mod_values   :=   ADDR(vphone);
    vphone[0]             :=   '1 801 555 1212';
    vphone[1]             :=   nil;

    modMail.mod_op        :=   LDAP_MOD_ADD;
    modMail.mod_type      :=   'mail';
    modMail.mod_values    :=   ADDR(vmail);
    vmail[0]              :=   'JSmith@Acme.com';
    vmail[1]              :=   nil;

    modupName.mod_op      :=   LDAP_MOD_ADD;                { active directory }
    modupName.mod_type    :=   'userPrincipalName';
    modupName.mod_values  :=   ADDR(vupname);
    vupname[0]            :=   'james@hs-ulm.de';
    vupname[1]            :=   nil;

    ///* The userPassword attribute must be present to allow the
    //   user to log in.  Setting the value of UserPassword to an
    //   empty string would allow the user to login without a password.
    //*/
    modPass.mod_op        :=   LDAP_MOD_REPLACE;
    modPass.mod_type      :=   'userPassword';
    modPass.mod_values    :=   ADDR(vpass);
    vpass[0]              :=   'newpassword.0';
    vpass[1]              :=   nil;

    modSAM.mod_op         :=   LDAP_MOD_REPLACE;            { active directory }
    modSAM.mod_type       :=   'sAMAccountName';
    modSAM.mod_values     :=   ADDR(vsam);
    vsam[0]               :=   'james';
    vsam[1]               :=   nil;

    mods[0] := ADDR(modClass);
    mods[1] := ADDR(modCN);
    mods[2] := ADDR(modGN);
    mods[3] := ADDR(modSN);
    mods[4] := ADDR(modPhone);
    mods[5] := ADDR(modMail);
    mods[6] := ADDR(modupName);
    mods[7] := ADDR(modPass);
    mods[8] := ADDR(modSAM);
    mods[9] := nil;

    ///*
    // * Construct the dn "cn=james" with the container name from the command line
    // */
    dn := 'cn=james,'+containerName;

    ///*
    // * Now add the object
    // */
    writeln('ldap_add_ext_s');
    writeln('dn: ',dn);

    rc := ldap_add_ext_s(ld,           //* LDAP session handle */
                         pchar8(dn),   //* dn of the object to be created */
                         ADDR(mods),   //* array of add options and values */
                         nil,          //* server control */
                         nil);         //* client control */

    // http://www.directory-info.com/LDAP/LDAPErrorCodes.html
    
    if ( rc <> LDAP_SUCCESS ) then begin
      writeln('ldap_add_ext_s: #',rc,' ',ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;

    writeln('Entry successfully added: ', dn );

    ldap_unbind_s(ld);

end.
