// cert_cb.inc

function cert_callback(pHandle: pointer):int32; stdcall;
label
   err;
var
   rc         : int32;
   callbackRc : int32;
   cert       : LDAPSSL_Cert;
begin
   callbackRc  := LDAPSSL_CERT_REJECT;
   cert.data   := nil;
   cert.length := 0;

   ///* ldapssl_get_cert allows an application to get the certificate in
   // * a buffer encoded in Base64 or DER format -  Applications could
   // * then save the certificate buffer to persistant storage (file,
   // * database, etc ).
   // */

   ///* Query the length of the buffer needed by first calling
   // * ldapssl_get_cert with the data field set to NULL (ie cert.data=NULL).
   // * Upon return the length field is set to the required size.
   // */
   rc := ldapssl_get_cert(pHandle,                  //* cert handle */
                          LDAPSSL_CERT_BUFFTYPE_DER,//* desired encoding */
                          cert);                    //* certificate structure */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert length failed');
      goto err;
   end;

   ///* Allocate necessary memory
   // */
   getmem(cert.data,cert.length);
   if (cert.data=nil) then begin
     writeln('Could not allocate buffer!');
     goto err;
   end;

   ///* Retrieve the certificate DER Encoded
   // */
   rc := ldapssl_get_cert(pHandle,                  //* cert handle */
                          LDAPSSL_CERT_BUFFTYPE_DER,//* desired encoding */
                          cert);                    //* certificate structure */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert failed');
      freemem(cert.data);
      goto err;
   end;

   callbackRc := LDAPSSL_CERT_ACCEPT;

   ///* Accept certificate
   // */
   rc := ldapssl_add_trusted_cert(addr(cert), LDAPSSL_CERT_BUFFTYPE_DER);
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_add_trusted_cert failed');
   end;
   freemem(cert.data);

err:
   result := callbackRc;
end; //-------------------------------------------------------------------------
