// modified for Active Directory
(* $Novell: modattrs.c,v 1.8 2003/05/12 12:25:02 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
   modattrs.c 
***************************************************************************
   Description: The modattrs.c sample shows how to modify the attributes 
                of the entry created with the addentry.c sample. 
***************************************************************************)
program modattrsx;

uses sysutils, ldap;

//static char usage[] =
//"\n Usage:   modattrs <host name> <port number> <login dn> <password>"
//"\n\t <modify dn> \n"
//"\n Example: modattrs acme.com 389 cn=admin,o=acme secret"
//"\n\t cn=james,ou=sales,o=acme\n";

const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
    pwlen = 32;

var
    ld         : hLDAP;
    version, ldapPort, rc: int32;
    ldapHost    : AnsiString;
    loginDN     : AnsiString;
    password    : AnsiString;
    modifyDN    : AnsiString;

    mailValues  : array[0..1] of pchar8;
    phoneValues1: array[0..1] of pchar8;
    phoneValues2: array[0..1] of pchar8;
    modDescription, modMail, modPhone1, modPhone2: LDAPMod;
    modify      : array[0..4] of pLDAPMod;

(*
function WideCharToString(Source: PWideChar): string;
function StringToWideChar(const Source: string; Dest: PWideChar; DestSize: Integer): PWideChar;
    widestring
*)
   wDesc : WideString;
   berValues: array[0..1] of pberval;
   DescBer: berval;
begin
    wDesc := 'Och Hnsler mlt';
    DescBer.bv_len:= Length(wDesc);
    DescBer.bv_val:= PWideChar(wDesc);

    if (paramcount<>5) then begin
      writeln('Usage:   modattrs <host name> <port number> <login dn> <password><modify dn>');
      writeln('    Example: modattrs acme.com 389 cn=admin,o=acme secret cn=james,ou=sales,o=acme');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);
    modifyDN     := paramstr(5);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    //* Initialize the LDAP session
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);
    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('LDAP session initialized');

    //* Bind to the server */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind successful');

    ///*
    // * To modify the attributes of an entry
    // *    1  Specify the modify actions
    // *    2. Specify the attribute name to be modified
    // *    3. Specify the value of the attribute
    // *    4. Add to an array of LDAPMod structures
    // *    5. Call ldap_modify_ext_s
    // */

    ///*
    // * LDAP_MOD_REPLACE succeeds whether the value already exists or not.
    // */
    modMail.mod_op     := LDAP_MOD_REPLACE;
    modMail.mod_type   := 'mail';
    modMail.mod_values := ADDR(mailValues);
    mailValues[0]      := 'jimmy@acme.com';
    mailValues[1]      := nil;

    ///*
    // * LDAP_MOD_ADD UNI-Code String.
    // */
    modDescription.mod_op     := LDAP_MOD_REPLACE+LDAP_MOD_BVALUES;
    modDescription.mod_type   := 'description';
    modDescription.mod_values := ADDR(berValues);
    berValues[0]      := ADDR(DescBer);
    berValues[1]      := nil;

    ///*
    // * LDAP_MOD_ADD gives an error if the value already exists.
    // */
    modPhone1.mod_op     := LDAP_MOD_ADD;
    modPhone1.mod_type   := 'telephoneNumber';
    modPhone1.mod_values := ADDR(phoneValues1);
    phoneValues1[0]      := '1 234 567 8910';
    phoneValues1[1]      := nil;

    ///*
    // * LDAP_MOD_DELETE will give an error if the value does not exist.
    // * In this example, we delete the telephone number we just added.
    // */
    modPhone2.mod_op     := LDAP_MOD_DELETE;
    modPhone2.mod_type   := 'telephoneNumber';
    modPhone2.mod_values := ADDR(phoneValues2);
    phoneValues2[0]      := '1 234 567 8910';
    phoneValues2[1]      := nil;


    modify[0] := ADDR(modMail);
    modify[1] := ADDR(modDescription);
    modify[2] := ADDR(modPhone1);
    modify[3] := ADDR(modPhone2);
    modify[4] := nil;

    //* Modify the attributes */
    rc := ldap_modify_ext_s(ld,              //* LDAP session handle */
                            pchar8(modifyDN), //* the object to modify */
                            ADDR(modify),    //* array of LDAPMod structures */
                            nil,             //* server controls */
                            nil);            //* client controls */

    if ( rc <> LDAP_SUCCESS ) then begin
      writeln('ldap_modify_ext_s: #',rc,' ',ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;

    writeln(modifyDN,' modified successfully.');

    ldap_unbind_s(ld);
end.
