// modified for Active Directory
(* $Novell: search.c,v 1.14 2003/05/12 13:02:50 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
   search.c 
***************************************************************************
   Description: The search.c sample searches the directory, sorts the 
                results by the sn attribute, and prints the results.
                
                Note:  For simplicity, this sample prints all values as 
                if they were strings.  See the searchBinary.c sample program
                for an example of handling both string and binary values.
***************************************************************************)
program search;

uses sysutils, ldap, lber;

//static char usage[] =
//"\n Usage:   search <host name> <port number> <login dn> <password>"
//"\n          <search base>\n"
//"\n Example: search Acme.com 389 cn=admin,o=Acme secret ou=Sales,o=Acme\n";

const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */

type
    ppa = array[0..$4000] of pchar8;
    pppa = ^ppa;
var
    ld         : hLDAP;
    version, ldapPort, i, rc, entryCount: int32;
    ldapHost    : AnsiString;
    loginDN     : AnsiString;
    password    : AnsiString;
    searchBase  : AnsiString;

    attribute, dn, sortAttribute: pchar8;
    values      : ppchar8;

    ber         : BerElement;

    searchResult, entry: LDAPMessage;

function strcmp(const string1,string2: pchar8): integer; stdcall;
{
return value	description
<0	string1 is less than string2
0	string1 is the same as string2
>0	string1 is greater than string2
}
begin
  result := strcomp(string1,string2);
end{ strcmp };

begin
    sortAttribute := 'sn';

    if (paramcount<>5) then begin
      writeln('Usage:   search <host name> <port number> <login dn> <password><search base>');
      writeln('    Example: search Acme.com 389 cn=admin,o=Acme secret ou=Sales,o=Acme');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);
    searchBase   := paramstr(5);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    //* Initialize the LDAP session
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);
    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('LDAP session initialized');

    //* Bind to the server */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind successful');

    //* Search the directory */
    writeln('ldap_search_ext_s');
    rc := ldap_search_ext_s(
                    ld,                    //* LDAP session handle */
                    pchar8(searchBase),    //* container to search */
                    LDAP_SCOPE_ONELEVEL,   //* search scope */
                    '(objectclass=user)',  //* search filter */
                    nil,                   //* return all attributes */
                    0,                     //* return attributes and values */
                    nil,                   //* server controls */
                    nil,                   //* client controls */
                    ADDR(timeOut),         //* search timeout */
                    LDAP_NO_LIMIT,         //* no size limit */
                    searchResult );        //* returned results */

    if (rc <> LDAP_SUCCESS) then begin
        writeln('ldap_search_ext_s: ', ldap_err2string(rc));
        ldap_msgfree(searchResult);
        ldap_unbind_s(ld);
        halt(1);
    end;

    //* client-sort */
    writeln('ldap_sort_entries');
    ldap_sort_entries(ld, searchResult, sortAttribute, strcmp);

    //* Go through the search results by checking entries */
    writeln('ldap_first_entry');
    entry := ldap_first_entry(ld, searchResult);
    while (entry<>nil) do begin
//    for (   entry   =   ldap_first_entry( ld, searchResult );
//            entry   !=  NULL;
//            entry   =   ldap_next_entry( ld, entry ) )

        dn := ldap_get_dn(ld, entry);
        if (dn <> nil) then begin
            writeln('   dn: ', dn);
            ldap_memfree(dn);
        end;

        attribute := ldap_first_attribute(ld, entry, ber);
        while (attribute<>nil) do begin
//        for (   attribute = ldap_first_attribute( ld, entry, &ber );
//                attribute != NULL;
//                attribute = ldap_next_attribute( ld, entry, ber ) )

            //* Get values and print.  Assumes all values are strings. */
            values := ldap_get_values(ld, entry, attribute);
            if (values <> nil) then begin
                i := 0;
                while (pppa(values)^[i] <> nil) do begin
                    writeln('        ',attribute, ' = ' , pppa(values)^[i]);
                    inc(i);
                end{while};
                ldap_value_free(values);
            end;
            ldap_memfree(attribute);

            attribute := ldap_next_attribute(ld, entry, ber);
        end{while};

        ber_free(ber, 0);

        entry := ldap_next_entry(ld, entry );
    end{while};

    writeln('ldap_count_entries');
    entryCount := ldap_count_entries(ld,searchResult);

    writeln('Search completed successfully.');
    writeln('    Entries  returned: ',entryCount);

    ldap_msgfree(searchResult);

    ldap_unbind_s(ld);

end.
