// modified for Active Directory
(* $Novell: setpass.c,v 1.7 2003/05/12 13:06:33 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
   setpass.c   (Requires admin privilege) 
***************************************************************************
   Description: The setpass.c sample shows how to set the password 
                of an entry.  We simply set the userPassword attribute 
                of the object.
                
                In eDirectory only an admin can set a password without 
                supplying the old password.  Consequently this method 
                works on any Novell LDAP server, but only when the caller 
                has admin privileges.  Users cannot change their own 
                password with this method.
                
                Also see the modpass.c sample which allows a user to 
                specify the old and new password.

***************************************************************************)
program setpass;
{
   works with Netware eDirectory (disable TLS)
   works not with Microsoft Active Directory (see setpassAD)
}
uses sysutils, ldap;

//static char usage[] =
//"\n Usage:   setpass <host name> <port number> <login dn> <password>"
//"\n          <modify dn> <new password>\n"
//"\n Example: setpass Acme.com 389 cn=admin,o=Acme secret"
//"\n          cn=james,ou=Sales,o=Acme newpass\n";

const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
var
    ld         : hLDAP;
    version, ldapPort, rc: int32;
    ldapHost   : AnsiString;
    loginDN    : AnsiString;
    password   : AnsiString;
    modifyDN   : AnsiString;
    newPassword: AnsiString;

    passValues : array[0..1] of pchar8;
    modSetpass : LDAPMod;
    modify     : array[0..1] of pLDAPMod;
begin
    if (paramcount<>6) then begin
      writeln('Usage:   setpass <host name> <port number> <login dn> <password> <modify dn> <new password>');
      writeln('    Example: setpass Acme.com 389 cn=admin,o=Acme secret cn=james,ou=Sales,o=Acme newpass');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);
    modifyDN     := paramstr(5);
    newPassword  := paramstr(6);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    //* Initialize the LDAP session
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);
    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;

    writeln('LDAP session initialized');

    //* Bind to the server */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind successful');

    //*
    // * To modify the attributes of an entry
    // *    1  Specify the modify action
    // *    2. Specify the attribute name to be modified
    // *    3. Specify the value of the attribute
    // *    4. Add to an array of LDAPMod structures
    // *    5. Call ldap_modify_ext_s
    // */

    //*
    // * LDAP_MOD_REPLACE replaces any existing values with the new value.
    // */
    modSetpass.mod_op     := LDAP_MOD_REPLACE;
    modSetpass.mod_type   := 'userPassword';
    modSetpass.mod_values := ADDR(passValues);
    passValues[0]         := pchar8(newPassword);
    passValues[1]         := nil;

    modify[0] := ADDR(modSetpass);
    modify[1] := nil;

    //* Modify the attribute */
    writeln('ldap_modify_ext_s');
    writeln('dn: ',modifyDN);

    rc := ldap_modify_ext_s(ld,              //* LDAP session handle */
                            pchar8(modifyDN),//* the object to modify */
                            ADDR(modify),    //* array of LDAPMod structures */
                            nil,             //* server controls */
                            nil);            //* client controls */

    if ( rc <> LDAP_SUCCESS ) then begin
      writeln('ldap_modify_ext_s: #',rc,' ',ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;

    writeln('Password modified for: ',modifyDN);

    ldap_unbind_s(ld);
end.

