// modified for Active Directory
(* $Novell: setpass.c,v 1.7 2003/05/12 13:06:33 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
   setpass.c   (Requires admin privilege) 
***************************************************************************
   Description: The setpass.c sample shows how to set the password 
                of an entry.  We simply set the userPassword attribute 
                of the object.
                
                In eDirectory only an admin can set a password without 
                supplying the old password.  Consequently this method 
                works on any Novell LDAP server, but only when the caller 
                has admin privileges.  Users cannot change their own 
                password with this method.
                
                Also see the modpass.c sample which allows a user to 
                specify the old and new password.

***************************************************************************)
program setpassAD;
{
   special for Microsoft Active directory

   ldap_modify_ext_s: #53 DSA is unwilling to perform -> password too short

KB269190: How To Change a Windows 2000 User's Password Through LDAP

The password is stored in the Active Directory on a user object in the
unicodePwd attribute. This attribute can be written under restricted conditions,
but it cannot be read. The attribute can only be modified; it cannot be added on
object creation or queried by a search. In order to modify this attribute, the
client must have a 128-bit Secure Socket Layer (SSL) connection to the server.
For this connection to be possible, the server must possess a server certificate
for a 128-bit RSA connection, the client must trust the certificate authority (CA)
that generated the server certificate, and both client and server must be capable
of 128-bit encryption.

The syntax of the unicodePwd attribute is octet-string; however, the directory
service expects that the octet-string will contain a UNICODE string (as the name
of the attribute indicates). This means that any values for this attribute
passed in LDAP must be UNICODE strings that are BER-encoded (Basic Encoding Rules)
as an octet-string. In addition, the UNICODE string must begin and end in quotes
that are not part of the desired password.

There are two possible ways to modify the unicodePwd attribute. The first is
similar to a normal "user change password" operation. In this case, the modify
request must contain both a delete and an add operation. The delete operation
must contain the current password with quotes around it. The add operation must
contain the desired new password with quotes around it.

The second way to modify this attribute is analogous to an administrator
resetting a password for a user. In order to do this, the client must bind as
a user with sufficient permissions to modify another user's password. This
modify request should contain a single replace operation with the new desired
password surrounded by quotes. If the client has sufficient permissions, this
password become the new password, regardless of what the old password was.
}
uses sysutils, ldap, ldap_ssl;

//static char usage[] =
//"\n Usage:   setpass <host name> <port number> <login dn> <password>"
//"\n          <modify dn> <new password>\n"
//"\n Example: setpass Acme.com 389 cn=admin,o=Acme secret"
//"\n          cn=james,ou=Sales,o=Acme newpass\n";

{$I cert_cb.inc}

const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
var
    ld         : hLDAP;
    version, ldapPort, rc: int32;
    ldapHost   : AnsiString;
    loginDN    : AnsiString;
    password   : AnsiString;
    modifyDN   : AnsiString;
    newPassword: AnsiString;
    wPassword  : WideString;

    passValues : array[0..1] of pberval;
    berPassword: berval;
    modSetpass : LDAPMod;
    modify     : array[0..1] of pLDAPMod;

begin
    if (paramcount<>6) then begin
      writeln('Usage:   setpass <host name> <port number> <login dn> <password> <modify dn> <new password>');
      writeln('    Example: setpass Acme.com 389 cn=admin,o=Acme secret cn=james,ou=Sales,o=Acme newpass');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);
    modifyDN     := paramstr(5);
    newPassword  := paramstr(6);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    //* works only with a SSL-connection
    writeln('ldapssl_client_init');
    rc := ldapssl_client_init(nil,         //* DER encoded cert file  */
                              nil);        //* reserved, use NULL */

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_client_init error: ', rc);
      halt(1);
    end;

    ///*
    // * Register a certificate verification callback routine.  This routine
    // * will be called when any untrusted server certificate is received
    // * when establishing an SSL connection.
    // */
    writeln('ldapssl_set_verify_callback');
    rc := ldapssl_set_verify_callback(cert_callback);

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_set_verify_callback error: ', rc);
    end;

    ///*
    //* create a LDAP session handle that is enabled for ssl connection
    //*/
    writeln('ldapssl_init');
    ld := ldapssl_init(pchar8(ldapHost),//* host name */
                       ldapPort,        //* port number */
                       1);              //* 0- clear text, 1-enable for ssl */

    if (ld = nil) then begin
      writeln('ldapssl_init error' );
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('LDAP session initialized');

    //* Bind to the server */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind successful');

    //*
    // * To modify the attributes of an entry
    // *    1  Specify the modify action
    // *    2. Specify the attribute name to be modified
    // *    3. Specify the value of the attribute
    // *    4. Add to an array of LDAPMod structures
    // *    5. Call ldap_modify_ext_s
    // */

    //*
    // * LDAP_MOD_REPLACE replaces any existing values with the new value.
    // */
    wPassword := '"'+newPassword+'"';
    berPassword.bv_len:= Length(wPassword)*sizeOf(widechar);
    berPassword.bv_val:= pwidechar(wpassword);

    modSetpass.mod_op     := LDAP_MOD_REPLACE+LDAP_MOD_BVALUES;
    modSetpass.mod_type   := 'unicodePwd';
    modSetpass.mod_values := ADDR(passValues);
    passValues[0]         := ADDR(berPassword);
    passValues[1]         := nil;

    modify[0] := ADDR(modSetpass);
    modify[1] := nil;

    //* Modify the attribute */
    writeln('ldap_modify_ext_s');
    writeln('dn: ',modifyDN);

    rc := ldap_modify_ext_s(ld,              //* LDAP session handle */
                            pchar8(modifyDN),//* the object to modify */
                            ADDR(modify),    //* array of LDAPMod structures */
                            nil,             //* server controls */
                            nil);            //* client controls */

    if ( rc <> LDAP_SUCCESS ) then begin
      writeln('ldap_modify_ext_s: #',rc,' ',ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;

    writeln('Password modified for: ',modifyDN);

    ldap_unbind_s(ld);

    ///*
    //* Uninitialize the LDAP ssl library
    //*/
    ldapssl_client_deinit();
end.

