(* $Novell: ssl_any.c,v 1.10 2003/05/12 13:08:29 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    ssl_any.c
***************************************************************************
   Description: Demonstrates making an ssl connection to an ldap server
      without any server certification verification.
***************************************************************************)
program ssl_any;

uses sysutils, ldap, ldap_ssl;

//static char usage[] =
//"\n Usage:   ssl_any <host name> <port number> <login dn> <password>\n"
//"  host name      = ldap server name or IP address\n"
//"  port number    = port to use - 636 is ldap ssl default\n"
//"  login dn       = user name to login as\n"
//"  password       = user password\n"
//"\n Example: ssl_any Acme.com 636 cn=admin,o=Acme secret\n";

const timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
var version, rc, ldapPort: int32;
    ldapHost  : String;
    loginDN   : String;
    password  : String;

    attrs     : array[0..1] of pchar;

    ld        : hLDAP;
    resultBuf : LDAPMessage;
    entry     : LDAPMessage;
    dn        : pchar;

begin

    if (paramcount()<>4) then begin
      writeln('Usage:   sslbind <host name> <port number> <login dn> <password>');
      writeln('  host name      = ldap server name or IP address');
      writeln('  port number    = port to use - 636 is ldap ssl default');
      writeln('  login dn       = user name to login as');
      writeln('  password       = user password');
      writeln('    Example: ssl_any Acme.com 636 cn=admin,o=Acme secret');
      halt(1);
    end;


    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);


    //*
    //* Initialize the ssl library.  The first parameter of
    //* ldapssl_client_init is a certificate file.  However, when used
    //* the file must be a DER encoded file.  NULL is passed in for the
    //* certificate file because ldapssl_set_verify_mode will be used
    //* to specify no server certificate verification.
    //*
    //* ldapssl_client_init is an application level initialization not a
    //* thread level initilization and should be done once.
    //*/
    writeln('ldapssl_client_init');
    rc := ldapssl_client_init(nil,         //* DER encoded cert file  */
                              nil);        //* reserved, use NULL */

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_client_init error: ', rc);
      halt(1);
    end;

    ///*
    //* Configure the LDAP SSL library to not verify the server certificate.
    //* The default is LDAPSSL_VERIFY_SERVER which validates all servers
    //* against the trusted certificates normally passed in
    //* during ldapssl_client_init or ldapssl_add_trusted_cert.
    //*
    //* WARNING:  Setting the verify mode to LDAPSSL_VERIFY_NONE turns off
    //* server certificate verification.  This means all servers are
    //* considered trusted.  This should be used only in controlled
    //* environments where encrypted communication between servers and
    //* clients is desired but server verification is not necessary.
    //*/
    writeln('ldapssl_set_verify_mode');
    rc := ldapssl_set_verify_mode(LDAPSSL_VERIFY_NONE);

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_set_verify_mode error: ', rc);
      ldapssl_client_deinit();
      halt(1);
    end;

    ///*
    //* create a LDAP session handle that is enabled for ssl connection
    //*/
    writeln('ldapssl_init');
    ld := ldapssl_init(pchar(ldapHost), //* host name */
                       ldapPort,        //* port number */
                       1);              //* 0- clear text, 1-enable for ssl */

    if (ld = nil) then begin
      writeln('ldapssl_init error' );
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar(loginDN), pchar(password));

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s error: ', rc, ' ',ldap_err2string( rc ));
      ldap_unbind_s(ld);
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('SSL bind successful - performing search');


    ///*
    //* Do a simple search just to verify things worked
    //*/
    attrs[0] := LDAP_NO_ATTRS;
    attrs[1] := nil;

    rc := ldap_search_ext_s(ld, 'ou=Benutzer,dc=hs-ulm,dc=de' , LDAP_SCOPE_ONELEVEL, '(objectClass=*)',
                           addr(attrs), 0, nil, nil, nil,
                           LDAP_NO_LIMIT, resultBuf);

    if (rc = LDAP_SUCCESS) then begin
       entry := ldap_first_entry(ld, resultBuf);

       while (entry <> nil) do begin
          dn := ldap_get_dn(ld, entry);
          writeln('Object: ', dn);

          ldap_memfree(dn);

          entry := ldap_next_entry(ld, entry);
       end;

       ldap_msgfree(resultBuf);
    end else begin
       writeln('ldap_search_s error: ', rc, ' ', ldap_err2string(rc));
    end;

    ldap_unbind_s( ld );

    ///*
    //* Uninitialize the LDAP ssl library
    //*/
    ldapssl_client_deinit();

end.

