(* $Novell: sslbind.c,v 1.16 2003/05/12 13:09:10 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    sslbind.c & sslbindi.c
***************************************************************************
   Description: Demonstrates connecting to a to an ldap server using SSL
***************************************************************************)
program sslbind2;

uses sysutils, ldap, ldap_ssl;

//static char usage[] =
//"\n Usage:   sslbind <host name> <port number> <login dn> <password>"
//"\n\t<cert file> <file type> \n"
//"  host name      = ldap server name or IP address\n"
//"  port number    = port to use - 636 is ldap ssl default\n"
//"  login dn       = user name to login as\n"
//"  password       = user password\n"
//"  cert file      = trusted root certificate file\n"
//"  file type      = DER=der encoded file, B64=b64 encoded file\n"
//"\n Example: sslbind Acme.com 636 cn=admin,o=Acme secret myKey.der DER\n";

function cert_callback(pHandle: pointer):int32; stdcall;
label
   err;
var
   rc         : int32;
   callbackRc : int32;
   cert       : LDAPSSL_Cert;
begin
   callbackRc  := LDAPSSL_CERT_REJECT;
   cert.data   := nil;
   cert.length := 0;

   ///* ldapssl_get_cert allows an application to get the certificate in
   // * a buffer encoded in Base64 or DER format -  Applications could
   // * then save the certificate buffer to persistant storage (file,
   // * database, etc ).
   // */

   ///* Query the length of the buffer needed by first calling
   // * ldapssl_get_cert with the data field set to NULL (ie cert.data=NULL).
   // * Upon return the length field is set to the required size.
   // */
   rc := ldapssl_get_cert(pHandle,                  //* cert handle */
                          LDAPSSL_CERT_BUFFTYPE_DER,//* desired encoding */
                          cert);                    //* certificate structure */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert length failed');
      goto err;
   end;

   ///* Allocate necessary memory
   // */
   getmem(cert.data,cert.length);
   if (cert.data=nil) then begin
     writeln('Could not allocate buffer!');
     goto err;
   end;

   ///* Retrieve the certificate DER Encoded
   // */
   rc := ldapssl_get_cert(pHandle,                  //* cert handle */
                          LDAPSSL_CERT_BUFFTYPE_DER,//* desired encoding */
                          cert);                    //* certificate structure */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert failed');
      freemem(cert.data);
      goto err;
   end;

   callbackRc := LDAPSSL_CERT_ACCEPT;

   ///* Accept certificate
   // */
   rc := ldapssl_add_trusted_cert(addr(cert), LDAPSSL_CERT_BUFFTYPE_DER);
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_add_trusted_cert failed');
   end;
   freemem(cert.data);

err:
   result := callbackRc;
end; //-------------------------------------------------------------------------

const timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
var version, rc, ldapPort: int32;
    ldapHost  : AnsiString;
    searchBase: AnsiString;
    loginDN   : AnsiString;
    password  : AnsiString;

    ld        : hLDAP;
    resultBuf : LDAPMessage;
    entry     : LDAPMessage;
    attrs     : array[0..1] of pchar8;
    dn        : pchar8;

begin
    if (paramcount()<>5) then begin
      writeln('Usage:   sslbind2 <host name> <port number> <search base> <login dn> <password> <cert file> <file type>');
      writeln('  host name      = ldap server name or IP address');
      writeln('  port number    = port to use - 636 is ldap ssl default');
      writeln('  search base    = the search base');
      writeln('  login dn       = user name to login as');
      writeln('  password       = user password');
      writeln('    Example: sslbind2 Acme.com 636 o=users cn=admin,o=Acme secret');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    searchBase   := paramstr(3);
    loginDN      := paramstr(4);
    password     := paramstr(5);


    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    ///*
    // * Initialize the ssl library.  The first parameter of
    // * ldapssl_client_init, if specified, must be a DER-encoded
    // * certificate file.  If NULL is given for this parameter, the
    // * certificate file may be specified by the ldapssl_add_trusted_cert
    // * API, which accepts either DER or B64-encoded files.
    // *
    // * ldapssl_client_init is an application level initialization not a
    // * thread level initilization and should be done once.
    // */

    writeln('ldapssl_client_init');
    rc := ldapssl_client_init(nil,         //* DER encoded cert file  */
                              nil);        //* reserved, use NULL */

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_client_init error: ', rc);
      halt(1);
    end;

    ///*
    // * Register a certificate verification callback routine.  This routine
    // * will be called when any untrusted server certificate is received
    // * when establishing an SSL connection.
    // */
    writeln('ldapssl_set_verify_callback');
    rc := ldapssl_set_verify_callback(cert_callback);

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_set_verify_callback error: ', rc);
    end;

    ///*
    // * create a LDAP session handle that is enabled for ssl connection
    // */
    writeln('ldapssl_init');
    ld := ldapssl_init(pchar8(ldapHost),//* host name */
                       ldapPort,        //* port number */
                       1);              //* 0- clear text, 1-enable for ssl */

    if (ld = nil) then begin
      writeln('ldapssl_init error' );
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s error: ', rc, ' ',ldap_err2string( rc ));
      ldap_unbind_s(ld);
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('SSL bind successful - performing search');


    ///*
    // * do a simple search just to verify things worked
    // */
    attrs[0] := LDAP_NO_ATTRS;
    attrs[1] := nil;

    writeln('ldap_search_ext_s');
    rc := ldap_search_ext_s(ld, pchar8(searchBase) , LDAP_SCOPE_ONELEVEL, '(objectClass=*)',
                           addr(attrs), 0, nil, nil, nil,
                           LDAP_NO_LIMIT, resultBuf);

    if (rc = LDAP_SUCCESS) then begin
       entry := ldap_first_entry(ld, resultBuf);

       while (entry <> nil) do begin
          dn := ldap_get_dn(ld, entry);
          writeln('Object: ', dn);

          ldap_memfree(dn);

          entry := ldap_next_entry(ld, entry);
       end;

       ldap_msgfree(resultBuf);
    end else begin
       writeln('ldap_search_s error: ', rc, ' ', ldap_err2string(rc));
    end;

    ldap_unbind_s(ld);

    ///*
    // * Uninitialize the LDAP ssl library
    // */
    ldapssl_client_deinit();
END.
