(* $Novell: sslbindi.c,v 1.11 2003/05/12 13:09:42 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    sslbindi.c
***************************************************************************
   Description: sslbindi.c demonstrates how to use the
                interactive certificate feature in LDAP SSL. The user code
                defines and registers a certificate verification callback
                routine. This callback is called if an unknown certificate
                is received while doing a connection to an LDAP server. The
                callback routine can retrieve information from the
                unknown certificate via helper functions.
***************************************************************************)
program sslbindi;

uses sysutils, ldap, ldap_ssl;

//static char usage[] =
//"\n Usage:   sslbindi <host name> <port number> <login dn> <password> "
//"\n"
//"\n Example: sslbindi Acme.com 636 cn=admin,o=Acme secret\n";

//int LIBCALL cert_callback
//(
//   void *pHandle
//);
(*********************************************************************
*
* This is the certificate verification callback routine.  A
* verification callback routine must return an int and have one
* void* parameter.
*
* The routine should return LDAPSSL_CERT_ACCEPT to continue with
* the SSL connection or LDAPSSL_CERT_REJECT to abort the SSL
* connection.
*
*********************************************************************)
function cert_callback(pHandle: pointer):int32; stdcall;
label
   err;
var
   rc         : int32;
   callbackRc : int32;
   length     : int32;
   certStatus : int32;
   answer     : AnsiString;
   value      : pchar8;
   cert       : LDAPSSL_Cert;
   certPeriod : LDAPSSL_Cert_Validity_Period;

begin
   value       := nil;
   callbackRc  := LDAPSSL_CERT_REJECT;

   cert.data   := nil;
   cert.length := 0;

   writeln;
   writeln('Certificate received.');
   writeln('Certificate Information:');

   ///*
   // * The following functions demonstrate retrieving information
   // * from the server certificate.
   // */

   ///* ldapssl_get_cert_attribute allows applications to query the
   // * size of a particular attribute by passing a NULL in the
   // * value parameter.  If a NULL is passed in, the length parameter
   // * is updated with the length of the data but no actual data is
   // * returned.  This allows applications to query an attribute's size
   // * and allocate appropriate memory for the value.
   // *
   // * NOTE: For attributes that are strings the length returned does not
   // * include the NULL termination character.
   // */

   ///* Certificate Status - status is an int
   // */

   length := sizeof(certStatus);
   writeln('ldapssl_get_cert_attribute');
   rc := ldapssl_get_cert_attribute(pHandle,                  //* cert Handle */
                                    LDAPSSL_CERT_GET_STATUS,  //* desired attribute */
                                    addr(certStatus),         //* attribute value */
                                    length);                  //* length */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert_attribute LDAPSSL_CERT_GET_STATUS failed');
      goto err;
   end;
   writeln('    Status:             ');

   case certStatus of
     UNABLE_TO_GET_ISSUER_CERT:
         writeln('unable to get issuer certificate');
     UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY:
         writeln('unable to decode issuer public key');
     CERT_SIGNATURE_FAILURE:
         writeln('certificate signature failure');
     CERT_NOT_YET_VALID:
         writeln('certificate is not yet valid');
     CERT_HAS_EXPIRED:
         writeln('Certificate has expired');
     ERROR_IN_CERT_NOT_BEFORE_FIELD:
         writeln('format error in certificate''s notBefore field');
     ERROR_IN_CERT_NOT_AFTER_FIELD:
         writeln('format error in certificate''s notAfter field');
     DEPTH_ZERO_SELF_SIGNED_CERT:
         writeln('self signed certificate');
     SELF_SIGNED_CERT_IN_CHAIN:
         writeln('self signed certificate in certificate chain');
     UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
         writeln('unable to get local issuer certificate');
     UNABLE_TO_VERIFY_LEAF_SIGNATURE:
         writeln('unable to verify the first certificate');
     INVALID_CA:
         writeln('invalid CA certificate');
     PATH_LENGTH_EXCEEDED:
         writeln('path length constraint exceeded');
     INVALID_PURPOSE:
         writeln('unsupported certificate purpose');
     CERT_UNTRUSTED:
         writeln('certificate not trusted');
     CERT_REJECTED:
         writeln('certificate rejected');
     else
         writeln(' number ',certStatus);
   end;

   ///* Certificate Issuer - attribute is a string
   // */

   ///* Query Certificate Issuer length
   // */
   writeln('ldapssl_get_cert_attribute (Query Certificate Issuer length)');
   rc := ldapssl_get_cert_attribute(pHandle,                 //* cert Handle */
                                    LDAPSSL_CERT_ATTR_ISSUER,//* desired attribute */
                                    nil,                     //* attribute value */
                                    length );                //* length */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert_attribute LDAPSSL_CERT_ATTR_ISSUER length failed');
      goto err;
   end;
   ///* add 1 for NULL termination
   // */
   inc(length);
   getmem(value,length);
   if (value=nil) then begin
      writeln('Could not allocate LDAPSSL_CERT_ATTR_ISSUER buffer!');
      goto err;
   end;

   ///* Retrieve the Issuer
   // */
   writeln('ldapssl_get_cert_attribute (Retrieve the Issuer)');
   rc := ldapssl_get_cert_attribute(pHandle,                 //* cert Handle */
                                    LDAPSSL_CERT_ATTR_ISSUER,//* desired attribute */
                                    value,                   //* attribute value */
                                    length);                 //* length */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert_attribute LDAPSSL_CERT_ATTR_ISSUER failed');
      goto err;
   end;
   writeln('    Issuer:             ',value);
   freemem(value,length);
   value := nil;


   ///* Certificate Subject - attribute is a string
   // */

   ///* Query Certificate Subject length
   // */
   writeln('ldapssl_get_cert_attribute (Query Certificate Subject length)');
   rc := ldapssl_get_cert_attribute(pHandle,                  //* cert Handle */
                                    LDAPSSL_CERT_ATTR_SUBJECT,//* desired attribute */
                                    nil,                      //* attribute value */
                                    length);                  //* length */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert_attribute LDAPSSL_CERT_ATTR_SUBJECT length failed');
      goto err;
   end;

   ///* add 1 for NULL termination
   // */
   inc(length);
   getmem(value,length);
   if (value=nil) then begin
      writeln('Could not allocate LDAPSSL_CERT_ATTR_SUBJECT buffer!');
      goto err;
   end;

   ///* Retrieve the Subject
   // */
   rc := ldapssl_get_cert_attribute(pHandle,                  //* cert Handle */
                                    LDAPSSL_CERT_ATTR_SUBJECT,//* desired attribute */
                                    value,                    //* attribute value */
                                    length);                  //* length */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert_attribute LDAPSSL_CERT_ATTR_SUBJECT failed');
      goto err;
   end;
   writeln('    Subject:            ', value);
   freemem(value);
   value := nil;

   ///* Certificate validity period - attribute is the struct
   // *    LDAPSSL_CERT_VALIDITY_PERIOD - because the value is of fixed
   // *    size there is no need to initially query for size.
   // */

   ///* Retrieve validity period.
   // */
   rc := ldapssl_get_cert_attribute(pHandle,
                                    LDAPSSL_CERT_ATTR_VALIDITY_PERIOD,
                                    addr(certPeriod),
                                    length );
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert_attribute LDAPSSL_CERT_ATTR_VALIDITY_PERIOD failed');
      goto err;
   end;

   ///* The validity period consists of a Not Before and Not After time
   // */
   write('    Not Before Time:    ',certPeriod.notBeforeTime,' Type: ');
   if (certPeriod.notBeforeType = LDAPSSL_CERT_UTC_TIME) then begin
     writeln('UTC Time');
   end else begin
     writeln('Generalized Time');
   end;

   write('    Not After Time:     ',certPeriod.notAfterTime,' Type: ');
   if (certPeriod.notAfterType = LDAPSSL_CERT_UTC_TIME) then begin
     writeln('UTC Time');
   end else begin
     writeln('Generalized Time');
   end;

   ///* ldapssl_get_cert allows an application to get the certificate in
   // * a buffer encoded in Base64 or DER format -  Applications could
   // * then save the certificate buffer to persistant storage (file,
   // * database, etc ).
   // */


   ///* Query the length of the buffer needed by first calling
   // * ldapssl_get_cert with the data field set to NULL (ie cert.data=NULL).
   // * Upon return the length field is set to the required size.
   // */
   cert.data := nil;
   rc := ldapssl_get_cert(pHandle,                  //* cert handle */
                          LDAPSSL_CERT_BUFFTYPE_DER,//* desired encoding */
                          cert);                    //* certificate structure */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert length failed');
      goto err;
   end;

   ///* Allocate necessary memory
   // */
   getmem(cert.data,cert.length);
   if (cert.data=nil) then begin
     writeln('Could not allocate buffer!');
     goto err;
   end;

   ///* Retrieve the certificate DER Encoded
   // */
   rc := ldapssl_get_cert(pHandle,                  //* cert handle */
                          LDAPSSL_CERT_BUFFTYPE_DER,//* desired encoding */
                          cert);                    //* certificate structure */
   if (rc <> LDAPSSL_SUCCESS) then begin
      writeln('ldapssl_get_cert failed');
      goto err;
   end;

   ///* Prompt to continue and accept the certificate or abort - if
   // * the user choses to accept the certificate, ldapssl_add_trusted_cert
   // * is used to insert the certificate into the list of trusted
   // * certificates.  After inserting the certificate into the list,
   // * it will be trusted and this callback will not be called
   // * to verify the certificate again.
   // */
   write('Would you like to accept the certificate? (Y/N): ');
   readln(answer);

   if (answer = 'Y') or (answer = 'y') then begin
      callbackRc := LDAPSSL_CERT_ACCEPT;

      writeln('ldapssl_add_trusted_cert');
      rc := ldapssl_add_trusted_cert(addr(cert), LDAPSSL_CERT_BUFFTYPE_DER);
      if (rc <> LDAPSSL_SUCCESS) then begin
         writeln('ldapssl_add_trusted_cert failed');
      end;
   end else begin
      callbackRc := LDAPSSL_CERT_REJECT;
   end;

err:
   ///* Free the certificate buffer
   // */
   if (cert.data<>nil) then begin
      freemem(cert.data);
   end;
   ///* Free value buffer
   // */
   if (value<>nil) then begin
      freemem(value);
   end;

   result := callbackRc;
end; //-------------------------------------------------------------------------

const timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */

var rc        : int32;
    version   : int32;
    ldapPort  : int32;

    answer    : AnsiString;

    ldapHost  : AnsiString;
    loginDN   : AnsiString;
    password  : AnsiString;

    ld        : hLDAP;

begin
    if (paramcount()<>4) then begin
      writeln('Usage:   sslbindi <host name> <port number> <login dn> <password>');
      writeln('    Example: sslbindi Acme.com 636 cn=admin,o=Acme secret');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);

    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    ///*
    // * Initialize the ssl library - No trusted root certificate file is
    // * given to force the call of the verification callback routine
    // * during the SSL handshake.  At callback time the application can
    // * decide to accept or reject the server's certificate.
    // */
    writeln('ldapssl_client_init');
    rc := ldapssl_client_init(nil,         //* Certificate File */
                              nil);        //* reserverd, just use NULL */

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_client_init error: ', rc);
      halt(1);
    end;

    ///*
    // * Register a certificate verification callback routine.  This routine
    // * will be called when any untrusted server certificate is received
    // * when establishing an SSL connection.
    // */
    writeln('ldapssl_set_verify_callback');
    rc := ldapssl_set_verify_callback(cert_callback);

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_set_verify_callback error: ', rc);
    end;

    repeat
       ///*
       // * Create an SSL enabled LDAP handle
       // */
       writeln('ldapssl_init');
       ld := ldapssl_init(pchar8(ldapHost),//* host name */
                          ldapPort,        //* port number */
                          1 );             //* 0- clear text, 1-enable for ssl */

       if (rc <> LDAP_SUCCESS) then begin
         writeln('ldapssl_init error: ');
         ldapssl_client_deinit();
         halt(1);
       end;

       ///*
       // * Bind (connect) to the LDAP server.
       // */
       writeln('ldap_simple_bind_s');
       rc := ldap_simple_bind_s(ld,pchar8(loginDN),pchar8(password));

       if (rc <> LDAP_SUCCESS) then begin
         writeln('ldap_simple_bind_s error: ',ldap_err2string(rc));
         ldap_unbind_s(ld);
         ldapssl_client_deinit();
         halt(1);
       end;

       writeln('SSL bind successful');

       ///*
       // * Unbind
       // */
       ldap_unbind_s(ld);

       write('Would you like bind again? (Y/N): ');
       readln(answer);

    until (answer <> 'Y') and (answer <> 'y');

    ///*
    // * Uninitialize the LDAP ssl library
    // */
    ldapssl_client_deinit();
end.

